/* *****************************************************************************
 *
 * Filename: GridNavigateAction.java
 *
 * Author:   Robert Thornburrow
 * Project:  CalendarMaker
 * Created:  31 January 2003
 *
 * Purpose:  Action class for managing focus in Containers that use GridLayout.
 *
 * (c) Copyright R.H.Thornburrow, 2003
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 *****************************************************************************/

package com.tsac.calmaker.beans;

import java.awt.Component;
import java.awt.Container;
import java.awt.GridLayout;
import java.awt.KeyboardFocusManager;
import java.awt.event.*;
import javax.swing.AbstractAction;

/**
 * Generic Action class for navigating round a Container with a GridLayout.
 * Instances are bound to keys (usually the cursor-keys) using the
 * Container's InputMap.  There are usually four actions, one each for
 * up, down, left and right.  This class is designed to that components may
 * be added to or removed from the grid and so that the grid may be resized.
 * It will even cope with the GridLayout being replaced by another
 * GridLayout instance and with grids which are not full.
 *
 * @author  Robert
 */
public class GridNavigateAction
    extends javax.swing.AbstractAction
{
    // Properties
    private Container _parent;
    private int _xinc;
    private int _yinc;
        

    public GridNavigateAction( Container parent, int xinc, int yinc )
    {
        _parent = parent;
        _xinc = xinc;
        _yinc = yinc;
    }
        
        
    public void actionPerformed( ActionEvent e )
    {
        // Who is current owner?
        Component owner = KeyboardFocusManager.getCurrentKeyboardFocusManager().getFocusOwner();

        // Cache some values for later
        Component[] buttons = _parent.getComponents();
        GridLayout layout = (GridLayout) _parent.getLayout();
        int rowCount = layout.getRows();
        int colCount = layout.getColumns();

        // Find the index of the current owner
        int current;
        for( current = 0; (current < buttons.length) && (buttons[current] != owner); current++ );

        // If owner is one of ours then figure out next component
        if( current < buttons.length )
        {
            int row = current / colCount;
            int col = current % colCount;

            row = (row + _yinc) % rowCount;
            col = (col + _xinc) % colCount;

            if( row < 0 ) row += rowCount;
            if( col < 0 ) col += colCount;

            int next = (row * colCount) + col;

            if( next < buttons.length )
                buttons[next].requestFocus();
        }
    }        
}
