/* *****************************************************************************
 *
 * Filename: ListCalendar.java
 *
 * Author:   Robert Thornburrow
 * Project:  CalendarMaker
 * Created:  17 December 2002
 *
 * Purpose:  Calendar bean which has a list-like form.
 *
 * (c) Copyright R.H.Thornburrow, 2003
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 *****************************************************************************/

package com.tsac.calmaker.beans;

import java.awt.*;
import java.awt.geom.*;


public class ListCalendar
    extends CalendarBase
{
	public ListCalendar()
	{
        setPreferredSize( new Dimension( 160, 300 ) );
	}

    
    // From CalendarBase
    public boolean isStartOfWeekSupported()  { return false; }
    
    // From CalendarBase
    public boolean isCellBorderSupported() { return true; }
    
    // From CalendarBase
    public boolean isDiarySupported() { return true; }

    
    private int _longestday;    // From font
    private int _longestdate;   // From font
    
    
    // From CalendarBase
    protected void calclayout()
    {
        _resized = false;
        
        int width = this.getWidth();
        int height = this.getHeight();

        _cellwidth  = width - _inset;
        _cellheight = height / (float) 31;
        _leftmargin = _inset;
            
        
        // reduce font size to fit width of a column (day names)
        String longestdaystr = " " + Constants.getLongestDay( this, _dayfont ) + " ";
        _actualdayfont = Constants.reduceFontSizeToFit( this, _dayfont, longestdaystr, (int) _cellwidth, (int) _cellheight );
            
        _dayfontmetrics = getFontMetrics( _actualdayfont );
        _longestday = _dayfontmetrics.stringWidth( longestdaystr );
                        
        // reduce font size to fit width of a column (dates)
        _actualdatefont = Constants.reduceFontSizeToFit( this, _datefont, "00", (int) _cellwidth, (int) _cellheight );
        _datefontmetrics = getFontMetrics( _actualdatefont );
        _longestdate = _datefontmetrics.stringWidth( "00" );
        
        _actualdiaryfont = _diaryfont;
        _diaryfontmetrics = getFontMetrics( _actualdiaryfont );
    }

    
    // From CalendarBase
    public void paint( Graphics og, double scale )
    {
        Graphics2D g = (Graphics2D) og;
        
        if( _resized  )
            calclayout();
                
        float width       = (float) (scale * this.getWidth());
        float height      = (float) (scale * this.getHeight());
        float cellwidth   = (float) (scale * _cellwidth);
        float cellheight  = (float) (scale * _cellheight);
        float leftmargin  = (float) (scale * _leftmargin);
        float longestdate = (float) (scale * _longestdate);
        float longestday  = (float) (scale * _longestday);
        
        Font dayfont   = _actualdayfont.deriveFont( (float) (scale * _actualdayfont.getSize()) );
        Font datefont  = _actualdatefont.deriveFont( (float) (scale * _actualdatefont.getSize()) );
        Font diaryfont = _actualdiaryfont.deriveFont( (float) (scale * _actualdiaryfont.getSize()) );
        FontMetrics dayfontmetrics   = this.getFontMetrics( dayfont );
        FontMetrics datefontmetrics  = this.getFontMetrics( datefont );
        FontMetrics diaryfontmetrics = this.getFontMetrics( diaryfont );

        float dayfontascent   = (float) (scale * _dayfontmetrics.getAscent());
        float datefontascent  = (float) (scale * _datefontmetrics.getAscent());
        float diaryfontascent = (float) (scale * _diaryfontmetrics.getAscent());
        
        Rectangle2D.Float rect;
        Line2D.Float line;
        
        rect = new Rectangle2D.Float( 0, 0, width, height );
        g.setColor( Color.white );
        g.fill( rect );
        
        float hpos = leftmargin;
        float vpos = 0;
                
        for( int row = 0; row < getDaysInMonth(); row++ )
        {
            // draw line above entry
            line = new Line2D.Float( hpos, vpos, hpos + cellwidth, vpos );
            g.setColor( _cellbordercolor );
            g.draw( line );
            
            int day = (_firstday + row) % 7;
            
            // draw date number    
            if( (day == 0) || (day == 6) )
                g.setColor( _cellwendtextcolor );
            else
                g.setColor( _celltextcolor );
                  
            g.setFont( datefont );
            g.drawString( "" + (row + 1), hpos + 2 /*_leftmargin*/, vpos + datefontascent );
            
            // draw day name
            g.setFont( dayfont );
            g.drawString( Constants.longdays[day+1], hpos + 2 /*_leftmargin*/ + longestdate + 12, vpos + dayfontascent );
            
            // draw diary entry
            if( _diaryEnabled && (_diary != null) && (_diary[row] != null) )
            {                        
                g.setColor( _celltextcolor );
                g.setFont( diaryfont );
                g.drawString( _diary[row].getWhat(), hpos + 2 /*_leftmargin*/ + longestdate + 12 + longestday + 12, vpos + diaryfontascent );
            }
            
            vpos += cellheight;
        }
    }
    
    ////////////////////////////////////////////////////////////////////////////
    // Accessibility stuff
    
    protected String getAccessibleName()
    {
        return resources.getString( "ListCalendar_Name" );
    }
    
    protected String getAccessibleDescription()
    {
        return resources.getString( "ListCalendar_Desc" );
    }    
}